<?php

/**
 * @file
 * CDT WebService singleton.
 * @usage get instance with: TmgmtCdtApi::getInstance();.
 */

/**
 * Class TmgmtCdtApi.
 */
class TmgmtCdtApi {
  private static $instance = NULL;

  private $baseUrl;
  private $proxyHost;
  private $proxyPort;
  private $proxyUser;
  private $proxyPass;
  private $proxyUserAgent;
  private $curlOptions;
  private $user;
  private $password;
  private $isReady;

  private $method;
  public $response;
  private $fakeResponse;
  private $hasDebug;
  private $originalActionPath;
  private $actionPath;
  private $url;
  private $paramsPath;
  private $paramsQuery;
  private $paramsBody;
  private $uniqid;

  private $cache;

  private $httpCodes = array(
    100 => 'Continue',
    101 => 'Switching Protocols',
    102 => 'Processing',
    103 => 'Checkpoint',
    200 => 'OK',
    201 => 'Created',
    202 => 'Accepted',
    203 => 'Non-Authoritative Information',
    204 => 'No Content',
    205 => 'Reset Content',
    206 => 'Partial Content',
    207 => 'Multi-Status',
    300 => 'Multiple Choices',
    301 => 'Moved Permanently',
    302 => 'Found',
    303 => 'See Other',
    304 => 'Not Modified',
    305 => 'Use Proxy',
    306 => 'Switch Proxy',
    307 => 'Temporary Redirect',
    400 => 'Bad Request',
    401 => 'Unauthorized',
    402 => 'Payment Required',
    403 => 'Forbidden',
    404 => 'Not Found',
    405 => 'Method Not Allowed',
    406 => 'Not Acceptable',
    407 => 'Proxy Authentication Required',
    408 => 'Request Timeout',
    409 => 'Conflict',
    410 => 'Gone',
    411 => 'Length Required',
    412 => 'Precondition Failed',
    413 => 'Request Entity Too Large',
    414 => 'Request-URI Too Long',
    415 => 'Unsupported Media Type',
    416 => 'Requested Range Not Satisfiable',
    417 => 'Expectation Failed',
    418 => 'I\'m a teapot',
    422 => 'Unprocessable Entity',
    423 => 'Locked',
    424 => 'Failed Dependency',
    425 => 'Unordered Collection',
    426 => 'Upgrade Required',
    449 => 'Retry With',
    450 => 'Blocked by Windows Parental Controls',
    500 => 'Internal Server Error',
    501 => 'Not Implemented',
    502 => 'Bad Gateway',
    503 => 'Service Unavailable',
    504 => 'Gateway Timeout',
    505 => 'HTTP Version Not Supported',
    506 => 'Variant Also Negotiates',
    507 => 'Insufficient Storage',
    509 => 'Bandwidth Limit Exceeded',
    510 => 'Not Extended',
  );

  /**
   * List of simulated api calls.
   *
   * @var array
   */
  private $simulatedCalls = array(
    'GET/RequestIdentifier',
    'GET/Requests/{requestyear}/{requestnumber}/targets',
    'GET/TranslationComplete/{requestyear}/{requestnumber}',
    'POST/Requests',
    'GET/Requests/{requestyear}/{requestnumber}',
    'POST/Requests/ValidateRequest',
  );

  /**
   * Production Apis calls accepted in simulated mode.
   *
   * @var array
   */
  private $simulatedAuthorisedProductionCalls = array();

  /**
   * Constructor with defaut values.
   */
  public function __construct() {
    $this->uniqid = uniqid();
    $this->apiMode = variable_get('tmgmt_cdt_api_mode', TMGMT_CDT_API_MODE);

    $this->baseUrl = variable_get('tmgmt_cdt_api_url');

    $this->proxyHost = variable_get('tmgmt_cdt_api_proxy_host');
    $this->proxyPort = variable_get('tmgmt_cdt_api_proxy_port');
    $this->proxyUser = variable_get('tmgmt_cdt_api_proxy_user');
    $this->proxyPass = variable_get('tmgmt_cdt_api_proxy_pass');
    $this->proxyUserAgent = variable_get('tmgmt_cdt_api_proxy_user_agent');

    $this->user = variable_get('tmgmt_cdt_api_user', '');
    $this->password = base64_decode(variable_get('tmgmt_cdt_api_password', ''));
    $this->isReady = variable_get('tmgmt_cdt_api_ready', TMGMT_CDT_API_READY);
    $this->cache = new stdClass();

    try {
      $this->curlOptions = drupal_json_decode(variable_get('tmgmt_cdt_api_curl_options', TMGMT_CDT_API_CURL_OPTIONS));
    }
    catch (\Exception $e) {
      variable_set('tmgmt_cdt_api_ready', FALSE);
    }

  }

  /**
   * Change mode to PRODUCTION or SIMULATION.
   */
  public function setMode($mode = 'SIMULATION') {
    $this->apiMode = $mode;
  }

  /**
   * Get mode: PRODUCTION or SIMULATION.
   */
  public function getMode() {
    return $this->apiMode;
  }

  /**
   * GetInstance.
   */
  public static function getInstance() {
    if (self::$instance === NULL) {
      self::$instance = new TmgmtCdtApi();
    }
    self::$instance->resetInstance();

    return self::$instance;
  }

  /**
   * Clean all variables.
   */
  public function resetInstance() {
    $this->apiMode = variable_get('tmgmt_cdt_api_mode', TMGMT_CDT_API_MODE);
    $this->response = new stdClass();
    $this->response->code = 400;
    $this->response->status = 'fail';
    $this->response->content = NULL;
    $this->response->error = "";
    $this->fakeResponse = NULL;
    $this->url = "";
    $this->hasDebug = FALSE;
    $this->method = "GET";
    $this->actionPath = "";
    $this->originalActionPath = "";
    $this->paramsPath = array();
    $this->paramsQuery = array();
    $this->paramsBody = new stdClass();
  }

  /**
   * SetDebug.
   */
  public function setDebug($state) {
    $this->hasDebug = (bool) $state;
  }

  /**
   * SetReady.
   */
  public function setReady($state) {
    $this->isReady = (bool) $state;
  }

  /**
   * SsetParamsPath: params by url paths.
   */
  public function setParamsPath($params) {
    $this->paramsPath = $params;
  }

  /**
   * SetParamsQuery: params by queryString.
   */
  public function setParamsQuery($params) {
    $this->paramsQuery = $params;
  }

  /**
   * SetParamsBody: params inside body (POST).
   */
  public function setParamsBody($params) {
    $this->paramsBody = $params;
  }

  /**
   * Use this response on next call.
   *
   * Ex format {'code': 200, 'status':'success', 'content': any}.
   */
  public function setFakeResponse($response) {
    $this->fakeResponse = $response;
  }

  /**
   * Return a custom response.
   */
  private function getFakeResponse() {

    $this->response->code = (int) $this->fakeResponse->code;
    $this->response->status = $this->fakeResponse->status === 'success' ? 'success' : 'fail';
    $this->response->content = $this->fakeResponse->content;
    $this->fakeResponse = NULL;

    return $this->response;

  }

  /**
   * Post.
   */
  public function post($actionPath) {
    $this->method = 'POST';
    $this->originalActionPath = $actionPath;
    return $this->call($actionPath);
  }

  /**
   * Get.
   */
  public function get($actionPath) {
    $this->method = 'GET';
    $this->originalActionPath = $actionPath;
    if (count($this->paramsPath) > 0) {
      foreach ($this->paramsPath as $id => $val) {
        $actionPath = str_replace('{' . trim($id) . '}', trim($val), $actionPath);
      }
    }
    return $this->call(trim($actionPath));
  }

  /**
   * Call.
   */
  private function call($actionPath) {
    try {

      if ($this->fakeResponse !== NULL) {
        return $this->getFakeResponse();
      }

      $this->setDebug(variable_get('tmgmt_cdt_api_debug', TMGMT_CDT_API_DEBUG));

      $this->actionPath = $actionPath;
      $this->url = $this->baseUrl;
      $this->url .= $actionPath;

      if (count($this->paramsQuery) > 0) {
        $this->url .= "?" . drupal_http_build_query($this->paramsQuery);
      }

      if ($this->hasDebug) {
        $this->response->debug = array(
          'uniqid' => $this->uniqid,
          'url' => $this->url,
          'originalActionPath' => $this->originalActionPath,
          'actionPath' => $this->actionPath,
          'simulatorUsed' => $this->useSimulator(),
          'method' => $this->method,
          'paramsQuery' => $this->paramsQuery,

          'paramsQueryJson' => drupal_json_encode($this->paramsQuery),
          'paramsBody' => $this->paramsBody,
          'paramsBodyJson' => drupal_json_encode($this->paramsBody),

          'user' => $this->user,
          'password' => $this->password,
          'proxyHost' => $this->proxyHost,
          'proxyPort' => $this->proxyPort,
          'proxyUser' => $this->proxyUser,
          'proxyPass' => $this->proxyPass,
          'proxyUserAgent' => $this->proxyUserAgent,
        );
      }

      if ($this->useSimulator()) {
        return $this->apiSimulator();
      }

      if (!$this->isReady) {
        throw new \Exception(t('Translator "@cdt_name" is not ready.', array('@cdt_name' => TMGMT_CDT_NAME)));
      }

            // @codingStandardsIgnoreStart
            // Do not use drupal_http_request, we cannot control all headers response.
            // https://api.drupal.org/comment/56628#comment-56628 
            $curl = curl_init();

            $curlopt[CURLOPT_USERPWD] = $this->user . ':' . $this->password;
            $curlopt[CURLOPT_HTTPHEADER]['Content-Type'] = 'application/json';
            $curlopt[CURLOPT_HTTPHEADER]['Accept'] = 'application/json';
            if ($this->method === 'POST') {
                // If params include html content, can be a problem using this function.
                $payload = drupal_json_encode($this->paramsBody);
                $curlopt[CURLOPT_POST] = TRUE;
                $curlopt[CURLOPT_POSTFIELDS] = $payload;
                $curlopt[CURLOPT_HTTPHEADER]['Content-Length'] = drupal_strlen($payload);
            }
            $curlopt[CURLOPT_URL] = $this->url;

            if (!empty($this->proxyHost)) {
                $curlopt[CURLOPT_PROXY] = $this->proxyHost;
                $curlopt[CURLOPT_PROXYPORT] = $this->proxyPort;
            }
            if (!empty($this->proxyUser)) {
              $curlopt[CURLOPT_PROXYUSERPWD] = $this->proxyUser . ':' . $this->proxyPass;
            }

            if (!empty($this->proxyUserAgent)) {
              $curlopt[CURLOPT_USERAGENT] = $this->proxyUserAgent;
            }

            foreach ( $this->curlOptions as $curl_option => $curl_value) {
              $curlopt[constant($curl_option)] = $curl_value;
            }

            curl_setopt_array($curl, $curlopt);

            // Curl main call.
            $raw_data = curl_exec($curl);

            // Get curl info for http code.
            $curl_info = curl_getinfo($curl);
            // @codingStandardsIgnoreEnd

      if ($this->hasDebug) {
        $this->response->debug['responseRaw'] = $raw_data;
        $this->response->debug['curlInfo'] = $curl_info;
        $this->response->debug['curlOptions'] = $curlopt;
      }

      $this->response->code = (int) $curl_info['http_code'];
      $curl_info = NULL;

      // Get curl error.
      if (curl_errno($curl)) {
        throw new \Exception(curl_error($curl));
      }

      $firewall_error = $this->isFirewallRejection($raw_data);
      if ($firewall_error !== FALSE) {
        throw new \Exception(t('Firewall error: @firewall_error', array('@firewall_error', $firewall_error)));
      }

      if (TmgmtCdtHelper::isJson($raw_data)) {
                // @codingStandardsIgnoreStart
                // Need a native json format.
                $this->response->content =  json_decode($raw_data);
                // @codingStandardsIgnoreEnd
      }
      else {
        $this->response->content = $raw_data;
      }
      unset($raw_data);

      if ($this->response->code !== 200) {
        if (!empty($this->response->content->Description)) {
          throw new \Exception($this->response->content->Description);
        }
        else {

          if (isset($this->httpCodes[$this->response->code])) {
            throw new \Exception($this->httpCodes[$this->response->code]);
          }
          else {
            throw new \Exception(t('Unknown error'));
          }
        }
      }

      $this->response->status = 'success';
      curl_close($curl);
      unset($curl);

      if ($this->hasDebug) {
        TmgmtCdtHelper::watchdogDebug($this->response->debug, 'API success');
      }
      $this->hasDebug = FALSE;

      return $this->response;
    }
    catch (\Exception $e) {

      $this->response->status = 'fail';
      $this->response->error = $e->getMessage();
      if ($this->hasDebug) {
        watchdog_exception('tmgmt_cdt', $e);
        TmgmtCdtHelper::watchdogDebug($this->response->debug, 'API fail');
      }

      if (isset($curl)) {
        @curl_close($curl);
        unset($curl);
      }

      $this->hasDebug = FALSE;
      return $this->response;
    }
  }

  /**
   * Check if internal simulator is used.
   */
  private function useSimulator() {

    if ($this->apiMode !== 'SIMULATION') {
      return FALSE;
    }

    $apiCall = $this->method . '/' . $this->originalActionPath;
    if (in_array($apiCall, $this->simulatedAuthorisedProductionCalls)) {
      return FALSE;
    }

    drupal_set_message(drupal_strtoupper(t('Simulator activated')), 'warning', FALSE);

    return TRUE;
  }

  /**
   * Simulated calls.
   */
  private function apiSimulator() {
    $apiCall = $this->method . '/' . $this->originalActionPath;

    if (!in_array($apiCall, $this->simulatedCalls)) {
      throw new Exception(t("Cannot simulate @api_call", array('@api_call' => $apiCall)));
    }

    if ($apiCall === 'GET/RequestIdentifier') {
      return $this->simulatedGetRequestIdentifier();
    }
    if ($apiCall === 'GET/TranslationComplete/{requestyear}/{requestnumber}') {
      return $this->simulatedGetTranslationCompleteRequestyearRequestnumber();
    }
    if ($apiCall === 'GET/Requests/{requestyear}/{requestnumber}/targets') {
      return $this->simulatedGetRequestsRequestyearRequestnumberTargets();
    }
    if ($apiCall === 'POST/Requests/ValidateRequest') {
      return $this->simulatedPostRequestsValidateRequest();
    }
    if ($apiCall === 'POST/Requests') {
      return $this->simulatedPostRequests();
    }
    if ($apiCall === 'GET/Requests/{requestyear}/{requestnumber}') {
      return $this->simulatedGetRequestsRequestyearRequestnumber();
    }
  }

  /**
   * Simulated GET /RequestIdentifier.
   */
  private function simulatedGetRequestIdentifier() {

    try {

      if (empty($this->paramsQuery['correlationID'])) {
        throw new Exception(t('correlationID is empty'));
      }

      $query = new EntityFieldQuery();
      $query
        ->entityCondition('entity_type', 'tmgmt_job')
        ->propertyCondition('translator', 'cdt')
        ->propertyCondition('cdt_correlation_id', $this->paramsQuery['correlationID']);
      $result = $query->execute();

      if (empty($result['tmgmt_job'])) {
        $this->response->status = 'fail';
        $this->response->error = t('Invalid correlation_id');
        return $this->response;
      }

      $ids = array_keys($result['tmgmt_job']);
      $job = entity_load_single(
        'tmgmt_job',
        reset($ids)
      );

      if (($job->cdt_request_identifier === TMGMT_CDT_DEFAUTL_REQUEST_IDENTIFIER) || empty($job->cdt_request_identifier)) {
        $this->response->content = date('Y') . '/' . $job->tjid . '-' . rand(1000, 9999);
      }
      else {
        $this->response->content = $job->cdt_request_identifier;
      }

      $this->response->code = 200;
      $this->response->status = 'success';

      return $this->response;
    }
    catch (\Exception $e) {
      $this->response->status = 'fail';
      $this->response->error = $e->getMessage();
      return $this->response;
    }
  }

  /**
   * Simulated GET /TranslationComplete/{requestyear}/{requestnumber}.
   */
  private function simulatedGetTranslationCompleteRequestyearRequestnumber() {
    $this->response->content = 'true';

    $this->response->code = 200;
    $this->response->status = 'success';

    return $this->response;
  }

  /**
   * Simulated GET /Requests/{requestyear}/{requestnumber}/target.
   */
  private function simulatedGetRequestsRequestyearRequestnumberTargets() {

    try {

      $request_identifier = $this->paramsPath['requestyear'] . '/' . $this->paramsPath['requestnumber'];
      $query = new EntityFieldQuery();
      $query
        ->entityCondition('entity_type', 'tmgmt_job')
        ->propertyCondition('translator', 'cdt')
        ->propertyCondition('cdt_request_identifier', $request_identifier);
      $result = $query->execute();

      if (empty($result['tmgmt_job'])) {
        $this->response->status = 'fail';
        $this->response->error = t('Invalid request_identifier');
        return $this->response;
      }

      $ids = array_keys($result['tmgmt_job']);
      $job = entity_load_single(
        'tmgmt_job',
        reset($ids)
      );

      $translator = $job->getTranslator();
      $job_source_language = $job->source_language;
            // @codingStandardsIgnoreStart
            // Need a native json format.
            $job_request_params = json_decode($job->cdt_request_params);
            // @codingStandardsIgnoreEnd
      $job_settings = $job->settings['cdt'];
      unset($job);

      $source_language = $translator->mapToRemoteLanguage($job_source_language);
      $target_languages = array();
      foreach (array_values($job_settings['target_languages']) as $lang) {
        $target_languages[] = $translator->mapToRemoteLanguage($lang);
      }

      $source_document_string = base64_decode($job_request_params->SourceDocuments[0]->File->FileDataBase64);
      $this->response->content = array();

      $count = 0;
      foreach ($target_languages as $target_language) {

        $count++;
        $dom_document = new DomDocument('1.0', 'UTF-8');
        libxml_use_internal_errors(TRUE);
        $dom_document->loadXML($source_document_string);

        $translation = new StdClass();

        $translation->SourceLanguage = $source_language;
        $translation->TargetLanguage = $target_language;
        $translation->FileData = "";
        $translation->FileDataBase64 = NULL;
        $translation->FileName = str_replace(".xml", "_$target_language.xml", $job_request_params->SourceDocuments[0]->File->FileName);
        $translation->ReferenceLanguages = NULL;

        $staticContents = $dom_document->getElementsByTagName("StaticContent");
        foreach ($staticContents as $staticContent) {

          $source_content = $staticContent->nodeValue;
          $staticContent->nodeValue = '';

          preg_match_all('/([^<>]*?)(<\/?[-:\w]+(?:>|\s[^<>]*?>)|$)/m', $source_content, $html_exploded, PREG_SET_ORDER, 0);
          $html_exploded_clean = array();
          $html_exploded_count = count($html_exploded);
          foreach ($html_exploded as $text) {

            // If html, after the first tag.
            if (($html_exploded_count > 2) && (count($html_exploded_clean) === 1)) {
              $text[0] = "($target_language) " . $text[0];
            }

            // If raw text, at the start.
            if (($html_exploded_count === 2) && (count($html_exploded_clean) === 0)) {
              $text[0] = "($target_language) " . $text[0];
            }

            $html_exploded_clean[] = $text[0];
          }

          $staticContent->setAttribute('state', 'translated');

          $translated_content = implode($html_exploded_clean);
          unset($html_exploded_clean);

          $staticContent->appendChild(
          $dom_document->createCDATASection($translated_content)
          );
        }
        $translation->FileData = base64_encode($dom_document->SaveXml());
        unset($dom_document);
        $this->response->content[] = $translation;
      }
      unset($source_document_string);

      $this->response->status = 'success';
      $this->response->code = 200;

      return $this->response;
    }
    catch (\Exception $e) {
      $this->response->status = 'fail';
      $this->response->error = $e->getMessage();
      return $this->response;
    }
  }

  /**
   * Simulated POST /Requests/ValidateRequest.
   */
  private function simulatedPostRequestsValidateRequest() {

    $this->response->content = new StdClass();

    $this->response->content->Errors = array();
    $this->response->content->OK = TRUE;

    $this->response->code = 200;
    $this->response->status = 'success';

    return $this->response;
  }

  /**
   * Simulated POST /Requests.
   */
  private function simulatedPostRequests() {
    try {

      $jobId = $this->paramsBody->ClientReference;
      $this->response->content = $jobId . '-' . rand(1000, 9999);

      $this->response->code = 200;
      $this->response->status = 'success';

      return $this->response;
    }
    catch (\Exception $e) {
      $this->response->status = 'fail';
      $this->response->error = $e->getMessage();
      return $this->response;
    }
  }

  /**
   * Simulated GET /Requests/{requestyear}/{requestnumber}.
   */
  private function simulatedGetRequestsRequestyearRequestnumber() {
    try {

      $request_identifier = $this->paramsPath['requestyear'] . '/' . $this->paramsPath['requestnumber'];
      $query = new EntityFieldQuery();
      $query
        ->entityCondition('entity_type', 'tmgmt_job')
        ->propertyCondition('translator', 'cdt')
        ->propertyCondition('cdt_request_identifier', $request_identifier);
      $result = $query->execute();

      if (empty($result['tmgmt_job'])) {
        $this->response->status = 'fail';
        $this->response->error = t('Invalid request_identifier');
        return $this->response;
      }

      $ids = array_keys($result['tmgmt_job']);
      $job = entity_load_single(
        'tmgmt_job',
        reset($ids)
      );

      $translator = $job->getTranslator();
      $businessReferenceData = $this->businessReferenceData();
            // @codingStandardsIgnoreStart
            // Need a native json format.
            $request_params = json_decode($job->cdt_request_params);
            // @codingStandardsIgnoreEnd
      $settings = $job->settings['cdt'];

      $this->response->content = new StdClass();
      $this->response->content->RequestIdentifier = $request_identifier;

      $status = 'COMPLETED';
      if (strpos(drupal_strtolower($job->defaultLabel()), 'cancelled') !== FALSE) {
        $status = 'CANCELLED';
        $job->setState(TMGMT_JOB_STATE_ABORTED);
        $job->save();
      }

      $this->response->content->Status = $status;
      $this->response->content->SourceLanguages = array($translator->mapToRemoteLanguage($job->source_language));

      foreach (array_values($job->getSetting('cdt')['target_languages']) as $lang) {
        $this->response->content->TargetLanguages[] = $translator->mapToRemoteLanguage($lang);
      }

      $this->response->content->CreationDate = (new DateTime())->setTimestamp($job->created)->format('c');
      $this->response->content->DeliveryDate = (new DateTime())->setTimestamp($job->changed)->format('c');
      $this->response->content->Title = $job->defaultLabel();
      $this->response->content->Service = 'Translation';

      foreach ($businessReferenceData->Departments as $dep) {
        if ($dep->Code === $settings['department']) {
          $this->response->content->Department = $dep->Description;
        }
      }

      $this->response->content->Contacts = array();
      foreach ($businessReferenceData->Contacts as $user) {
        foreach (array_values($settings['contacts']) as $id) {
          if ($user->UserName === $id) {
            $this->response->content->Contacts[] = $user->FirstName . ' ' . $user->LastName;
          }
        }
      }

      $this->response->content->DeliverToContacts = array();
      foreach ($businessReferenceData->Contacts as $user) {
        foreach (array_values($settings['contacts']) as $id) {
          if ($user->UserName === $id) {
            $this->response->content->DeliverToContacts[] = $user->FirstName . ' ' . $user->LastName;
          }
        }
      }

      $this->response->content->DocumentNames = array();
      $this->response->content->DocumentNames = (string) $request_params->SourceDocuments[0]->File->FileName;

      $this->response->code = 200;
      $this->response->status = 'success';

      return $this->response;
    }
    catch (\Exception $e) {
      $this->response->status = 'fail';
      $this->response->error = $e->getMessage();
      return $this->response;
    }
  }

  /**
   * Check if translations are available or not.
   */
  public function isTranslationComplete($requestyear, $requestnumber) {
    try {

      $params = array(
        'requestyear' => $requestyear,
        'requestnumber' => $requestnumber,
      );
      $this->setParamsPath($params);
      $this->get('TranslationComplete/{requestyear}/{requestnumber}');

      if ($this->response->status === 'success') {
        // Return always a boolean TRUE or FALSE.
        return drupal_strtolower($this->response->content) === 'true';
      }
      else {
        return FALSE;
      }
    }
    catch (\Exception $e) {
      return FALSE;
    }
  }

  /**
   * GetTranslations.
   */
  public function getTranslations($requestyear, $requestnumber) {
    try {

      $params = array(
        'requestyear' => $requestyear,
        'requestnumber' => $requestnumber,
      );
      $this->setParamsPath($params);
      $this->get('Requests/{requestyear}/{requestnumber}/targets');

      if ($this->response->status === 'success') {
        // Return an array, sometimes empty.
        return $this->response->content;
      }
      else {
        return FALSE;
      }
    }
    catch (\Exception $e) {
      return FALSE;
    }
  }

  /**
   * GetRequestIdentifier.
   */
  public function getRequestIdentifier($correlationID) {
    try {

      $params = array(
        'correlationID' => $correlationID,
      );
      $this->setParamsQuery($params);
      $this->get('RequestIdentifier');

      if ($this->response->status === 'success') {
        // example: "2019/123456".
        return (string) $this->response->content;
      }
      else {
        return FALSE;
      }
    }
    catch (\Exception $e) {
      return FALSE;
    }
  }

  /**
   * CheckConnection.
   */
  public function checkConnection() {
    try {

      if (!empty($this->cache->checkConnection)) {
        $this->response->code = 200;
        $this->response->status = 'success';
        return $this->cache->checkConnection;
      }

      $this->get('CheckConnection');
      $check_connection = $this->response->status === 'success';

      $this->cache->checkConnection = clone $check_connection;
      return $this->cache->checkConnection;
    }
    catch (\Exception $e) {
      return FALSE;
    }
  }

  /**
   * IsFirewallRejection.
   */
  private function isFirewallRejection($data) {

    if (!is_string($data)) {
      return FALSE;
    }
    preg_match_all('/(The requested URL was rejected. Please consult with your administrator.)|(Your support ID is: (\d*))/ims', $data, $matchs, PREG_SET_ORDER, 0);
    if (isset($matchs[0][0]) && isset($matchs[1][0])) {
      return $matchs[0][0] . ' ' . $matchs[1][0];
    }
    return FALSE;
  }

  /**
   * ValidateRequest.
   */
  public function validateRequest($request, $show_errors = TRUE) {
    try {

      if ($this->apiMode === 'PRODUCTION') {
        $show_errors = FALSE;
      }

      $this->setParamsBody($request);
      $this->post('Requests/ValidateRequest');

      if (
        $show_errors &&
        !$this->response->content->OK &&
        $this->response->status === 'success' &&
        isset($this->response->content->Errors)
      ) {
        foreach ($this->response->content->Errors as $error) {
          drupal_set_message($error->Description, 'error', FALSE);
        }
      }

      if ($show_errors && !$this->response->content->OK && $this->response->status !== 'success') {
        drupal_set_message($this->response->error, 'error', FALSE);
      }

      if ($this->response->status !== 'success') {
        return FALSE;
      }

      return $this->response->content->OK;
    }
    catch (\Exception $e) {
      return FALSE;
    }
  }

  /**
   * ClearCache.
   */
  public function clearCache() {
    $this->cache = new stdClass();
  }

  /**
   * BusinessReferenceData.
   */
  public function businessReferenceData() {
    try {
      if ($this->fakeResponse !== NULL) {
        return $this->getFakeResponse();
      }

      return variable_get('tmgmt_cdt_business_reference_data', new stdClass());

    }
    catch (\Exception $e) {
      return new stdClass();
    }
  }

}
