<?php

/**
 * @file
 * Contains job entity class.
 */

/**
 * Entity class for the tmgmt_job entity.
 */
class TMGMTCdtJob extends TMGMTJob {

  // @codingStandardsIgnoreStart
  // Need these properties names.
  public $cdt_correlation_id;
  public $cdt_request_identifier;
  public $cdt_request_params;
  // @codingStandardsIgnoreEnd

  /**
   * Clones job as unprocessed.
   */
  public function cloneAsUnprocessed() {
    $clone = parent::cloneAsUnprocessed();
    $clone->cdt_correlation_id = NULL;
    $clone->cdt_request_identifier = NULL;
    $clone->cdt_request_params = NULL;
    return $clone;
  }

  /**
   * Set a label.
   */
  public function defaultLabel() {

    if (isset($this->label) && !empty($this->label)) {
      return $this->label;
    }

    return t('Translation job #@tjid', array('@tjid' => $this->tjid));
  }

  /**
   * Adds an item to the translation job.
   */
  public function addItem($plugin, $item_type, $item_id) {

    $transaction = db_transaction();
    $is_new = FALSE;

    if (empty($this->tjid)) {
      $this->save();
      $is_new = TRUE;
    }

    $item = tmgmt_job_item_create($plugin, $item_type, $item_id, array(
      'tjid' => $this->tjid,
      'cdt_source_language' => $this->source_language,
    ));
    $item->save();

    if ($item->getWordCount() < 1) {
      $transaction->rollback();

      // In case we got word count 0 for the first job item, NULL tjid so that
      // if there is another addItem() call the rolled back job object will get
      // persisted.
      if ($is_new) {
        $this->tjid = NULL;
      }

      throw new TMGMTException(
        'Job item "@label" @type has no translatable content.',
        array('@label' => $item->label(), '@type' => $item->getSourceType())
      );
    }

    return $item;
  }

  /**
   * Create all job items.
   */
  public function jobItemGenerateTargetLanguages() {

    $target_languages = array_values($this->getSetting('cdt')['target_languages']);

    foreach ($this->getItems() as $job_item) {

      for ($i = 0; $i < count($target_languages); $i++) {
        if ($i === 0) {
          // First lang for the current item.
          $job_item->cdt_target_language = $target_languages[$i];
          $job_item->save();
        }
        else {
          // Create duplicates items for other languages.
          $clone_job_item = array();
          $clone_job_item['tjid'] = $job_item->tjid;
          $clone_job_item['data'] = $job_item->data;
          $clone_job_item['cdt_source_language'] = $job_item->cdt_source_language;
          $clone_job_item['cdt_target_language'] = $target_languages[$i];
          $clone_job_item['cdt_character_count'] = $job_item->cdt_character_count;
          $clone_job_item['cdt_settings'] = $job_item->cdt_settings;

          $item = tmgmt_job_item_create($job_item->plugin, $job_item->item_type, $job_item->item_id, $clone_job_item);
          $item->save();
        }
      }
    }
  }

  /**
   * Is not abordable.
   */
  public function isAbortable() {
    return FALSE;
  }

  /**
   * Is not deletable.
   */
  public function isDeletable() {
    return FALSE;
  }

  /**
   * Delete controled.
   */
  public function delete() {

    if (drupal_substr($this->defaultLabel(), 0, 6) === 'DELETE') {
      return parent::delete();
    }

    if (!$this->isFinished()) {
      parent::delete();
    }
    else {
      drupal_set_message(t('Cannot delete the finished job @title', array('@title' => $this->defaultLabel())), 'warning');
    }
  }

  /**
   * Add more indicators.
   */
  public function getCounts() {
    $count = array();
    foreach ($this->getItems() as $item) {
      if (!isset($count[$item->cdt_target_language][$item->item_id])) {
        $count[$item->cdt_target_language][$item->item_id] = 0;
      }
      $count[$item->cdt_target_language][$item->item_id] += $item->getCharacterCount();
    }

    if (count($count) > 0) {
      $languages = count(array_keys($count));
      $characters_language = (int) array_sum(reset($count));
      $pages_language = ceil($characters_language / 750) * 0.5;
      $items = count(reset($count));
    }
    else {
      $languages = 0;
      $characters_language = 0;
      $pages_language = 0;
      $items = 0;

    }

    $out = array(
      'languages' => $languages,
      'characters' => $characters_language * $languages,
      'characters_language' => $characters_language,
      'items' => $items,
      'pages' => $pages_language * $languages,
      'pages_language' => $pages_language,
    );

    return $out;
  }

  /**
   * Override parent function.
   */
  public function addTranslatedData($data, $key = NULL, $target_language = '') {

    $key = tmgmt_ensure_keys_array($key);

    $this->target_language = $target_language;
    $this->save();

    $items = $this->getItems();
    // If there is a key, get the specific item and forward the call.
    if (!empty($key)) {
      $item_id = array_shift($key);
      if (isset($items[$item_id]) && $items[$item_id]->getTargetLanguage() === $target_language) {

        $items[$item_id]->addTranslatedData($data, $key);
      }
    }
    else {
      foreach ($data as $key => $value) {
        if (isset($items[$key]) && $items[$key]->getTargetLanguage() === $target_language) {

          $items[$key]->addTranslatedData($value);
        }
      }
    }
  }

  /**
   * Request the translation of a job from the translator.
   */
  public function requestTranslation() {

    if (!$this->isTranslatable() || !$controller = $this->getTranslatorController()) {
      return FALSE;
    }
    $controller->requestTranslation($this);
  }

}
