<?php

/**
 * @file
 * Contains job item entity class.
 */

/**
 * Entity class for the tmgmt_job entity.
 */
class TMGMTCdtJobItem extends TMGMTJobItem {

  // @codingStandardsIgnoreStart
  // Need these properties names.
  public $cdt_source_language;
  public $cdt_target_language;
  public $cdt_character_count = 0;
  public $cdt_settings;
  // @codingStandardsIgnoreEnd

  /**
   * Clones as active.
   */
  public function cloneAsActive() {

    $clone = parent::cloneAsActive();
    $clone->cdt_source_language = NULL;
    $clone->cdt_target_language = NULL;
    $clone->cdt_character_count = NULL;
    $clone->cdt_settings = NULL;
    return $clone;
  }

  /**
   * Override parent function.
   */
  protected function count(&$item) {
    parent::count($item);
    if (!empty($item['#text'])) {
      if (_tmgmt_filter_data($item)) {
        $this->cdt_character_count += tmgmt_cdt_character_count($item['#text']);
        $this->cdt_settings['links'] = tmgmt_cdt_extract_hrefs($item['#text']);
      }
    }
  }

  /**
   * Recalculate statistical word-data: pending, translated, reviewed, accepted.
   */
  public function recalculateStatistics() {

    if (!$this->isAccepted()) {
      $this->cdt_character_count = 0;
    }
    parent::recalculateStatistics();
  }

  /**
   * Getter.
   */
  public function getTargetLanguage() {
    return $this->cdt_target_language;
  }

  /**
   * Getter.
   */
  public function getCharacterCount() {
    return (int) $this->cdt_character_count;
  }

  /**
   * Getter.
   */
  public function getSetting($name) {
    if (isset($this->cdt_settings[$name])) {
      return $this->cdt_settings[$name];
    }
  }

  /**
   * Loads the structured source data array from the source.
   */
  public function getSourceData() {

    try {

      $data = parent::getSourceData();
      $translatable_fields = variable_get('tmgmt_cdt_translatable_fields', unserialize(TMGMT_CDT_TRANSLATABLE_FIELDS));

      if (!isset($translatable_fields['node'])) {
        throw new Exception(
        t('Please check your <a href="@url" target="_blank">settings</a>.', array('@url' => url('admin/config/regional/tmgmt_cdt/content'))));
      }

      if ($this->item_type === 'node') {
        $node = node_load((int) $this->item_id);

        if (!in_array($node->type, array_keys($translatable_fields['node']))) {
          throw new Exception(
              t('The content "@title" (@type) has no translatable fields. Please check your <a href="@url" target="_blank">settings</a>.',
              array(
                '@title' => $node->title,
                '@type' => $node->type,
                '@url' => url('admin/config/regional/tmgmt_cdt/content'),
              )));
        }

        foreach (array_keys($data) as $field) {
          if (!in_array($field, array_keys($translatable_fields['node'][$node->type]))) {
            unset($data[$field]);
          }
          else {
            if ($translatable_fields['node'][$node->type][$field] === 0) {
              unset($data[$field]);
            }
          }
        }

        if (count($data) === 0) {
          throw new Exception(
              t('The content "@title" (@type) has no translatable fields. 
              Please check your <a href="@url" target="_blank">settings</a>.',
              array(
                '@title' => $node->title,
                '@type' => $node->type,
                '@url' => url('admin/config/regional/tmgmt_cdt/content'),
              )));
        }
        unset($node);
      }

      return $data;

    }
    catch (\Exception $e) {
      drupal_set_message($e->getMessage(), 'warning');
      return array();
    }

  }

  /**
   * Override parent function.
   */
  public function addTranslatedData($translation, $key = array()) {

    $this->addTranslatedDataRecursive($translation, $key);

    if ($this->isActive()) {
      $data = tmgmt_flatten_data($this->getData());

      $data = array_filter($data, '_tmgmt_filter_data');

      $finished = TRUE;
      foreach ($data as $item) {
        if (empty($item['#status']) || $item['#status'] === TMGMT_DATA_ITEM_STATE_PENDING) {
          $finished = FALSE;
          break;
        }
      }
      if ($finished) {
        $this->needsReview(FALSE);
        $uri = $this->uri();
        $job_uri = $this->getJob()->uri();
        $variables = array(
          '!source' => l($this->getSourceLabel(), $uri['path']),
          '@language' => $this->cdt_target_language,
          '!review_url' => url($uri['path'], array('query' => array('destination' => $job_uri['path']))),
        );
        $this->needsReview('The translation of !source to @language is finished and can now be <a href="!review_url">reviewed</a>.', $variables);
      }
    }
    $this->save();
  }

  /**
   * Override parent function.
   */
  public function acceptTranslation() {

    try {

      $job = $this->getJob();
      // To not break the tmgmt logic (source and target language per job)
      $job->target_language = $this->getTargetLanguage();
      $job->save();

      parent::acceptTranslation();
      $this->save();

      $message = t('Translation <strong>@language</strong> accepted for item <strong>@source</strong>.');
      $variables = array('@source' => $this->getSourceLabel(), '@language' => tmgmt_language_label($this->getTargetLanguage()));

      $this->setState(TMGMT_JOB_ITEM_STATE_ACCEPTED);
      $this->addMessage($message, $variables);

      if (tmgmt_job_check_finished($this->tjid)) {
        // If all item_job are done, mark the main job as finished.
        $job->finished();
      }

      // And back to multiple.
      $job->target_language = 'multiple';
      $job->save();

    }
    catch (\Exception $e) {
      drupal_set_message(t('Error accepting translation of Job jobItem @tjid: @error',
        array(' @tjid' => $this->tjid, '@error' => $e->getMessage())), 'error');
    }

  }

  /**
   * Override parent function.
   */
  protected function addTranslatedDataRecursive($translation, $key = array()) {

    if (isset($translation['#text'])) {
      $data = $this->getData(tmgmt_ensure_keys_array($key));

      if (empty($data['#status']) || $data['#status'] != TMGMT_DATA_ITEM_STATE_ACCEPTED) {

        // In case the origin is not set consider it to be remote.
        if (!isset($translation['#origin'])) {
          $translation['#origin'] = 'remote';
        }

        // If we already have a translation text and it hasn't changed, don't
        // update anything unless the origin is remote.
        if (!empty($data['#translation']['#text']) && $data['#translation']['#text'] === $translation['#text'] && $translation['#origin'] != 'remote') {
          return;
        }

        // In case the timestamp is not set consider it to be now.
        if (!isset($translation['#timestamp'])) {
          $translation['#timestamp'] = REQUEST_TIME;
        }
        // If we have a translation text and is different from new one create
        // revision.
        if (!empty($data['#translation']['#text']) && $data['#translation']['#text'] != $translation['#text']) {

          // Copy into $translation existing revisions.
          if (!empty($data['#translation']['#text_revisions'])) {
            $translation['#text_revisions'] = $data['#translation']['#text_revisions'];
          }

          // If current translation was created locally and the incoming one is
          // remote, do not override the local, just create a new revision.
          if (isset($data['#translation']['#origin']) && $data['#translation']['#origin'] === 'local' && $translation['#origin'] === 'remote') {
            $translation['#text_revisions'][] = array(
              '#text' => $translation['#text'],
              '#origin' => $translation['#origin'],
              '#timestamp' => $translation['#timestamp'],
            );
            $this->addMessage('Translation for customized @key received. Revert your changes if you wish to use it.', array('@key' => tmgmt_ensure_keys_string($key)));
            // Unset text and origin so that the current translation does not
            // get overridden.
            unset($translation['#text'], $translation['#origin'], $translation['#timestamp']);
          }
          // Do the same if the translation was already reviewed and origin is
          // remote.
          elseif ($translation['#origin'] === 'remote' && !empty($data['#status']) && $data['#status'] === TMGMT_DATA_ITEM_STATE_REVIEWED) {
            $translation['#text_revisions'][] = array(
              '#text' => $translation['#text'],
              '#origin' => $translation['#origin'],
              '#timestamp' => $translation['#timestamp'],
            );
            $this->addMessage('Translation for already reviewed @key received and stored as a new revision. Revert to it if you wish to use it.', array('@key' => tmgmt_ensure_keys_string($key)));
            // Unset text and origin so that the current translation does not
            // get overridden.
            unset($translation['#text'], $translation['#origin'], $translation['#timestamp']);
          }
          else {
            $translation['#text_revisions'][] = array(
              '#text' => $data['#translation']['#text'],
              '#origin' => isset($data['#translation']['#origin']) ? $data['#translation']['#origin'] : 'remote',
              '#timestamp' => isset($data['#translation']['#timestamp']) ? $data['#translation']['#timestamp'] : $this->changed,
            );
            // Add a message if the translation update is from remote.
            if ($translation['#origin'] === 'remote') {
              $diff = drupal_strlen($data['#translation']['#text']) - drupal_strlen($translation['#text']);
              $this->addMessage('Updated translation for key @key, size difference: @diff characters.', array('@key' => tmgmt_ensure_keys_string($key), '@diff' => $diff));
            }
          }
        }

        $values = array(
          '#translation' => $translation,
          '#status' => TMGMT_DATA_ITEM_STATE_TRANSLATED,
        );

        $this->updateData($key, $values);
      }
      return;
    }

    foreach (element_children($translation) as $item) {
      $this->addTranslatedDataRecursive($translation[$item], array_merge($key, array($item)));
    }
  }

}


/**
 * Extended class.
 */
// @codingStandardsIgnoreStart
// Using same prefix as module name.
class tmgmt_cdt_handler_field_tmgmt_job_item_operations extends views_handler_field_entity {
// @codingStandardsIgnoreEnd

  /**
   * Render.
   */
  public function render($values) {
    $item = $this->get_value($values);
    $element = array();
    $element['#theme'] = 'links';
    $element['#attributes'] = array('class' => array('inline'));
    $uri = $item->uri();
    if ($item->getCountTranslated() > 0 && entity_access('accept', 'tmgmt_job_item', $item)) {
      $element['#links']['review'] = array(
        'href' => $uri['path'],
        'query' => array('destination' => current_path()),
        'title' => t('review'),
      );
    }

    // Do not display view on unprocessed jobs.
    if (!$item->getJob()->isUnprocessed()) {
      $element['#links']['view'] = array(
        'href' => $uri['path'],
        'query' => array('destination' => current_path()),
        'title' => t('view'),
      );
    }
    return drupal_render($element);
  }

}

/**
 * Extended class.
 */
// @codingStandardsIgnoreStart
// Using same prefix as module name.
class tmgmt_cdt_handler_field_tmgmt_job_item_label extends views_handler_field_entity {
// @codingStandardsIgnoreEnd

  /**
   * Render.
   */
  public function render($values) {
    $item = $this->get_value($values);
    $uri = $item->getSourceUri();
    if (!empty($uri)) {
      $element = array();
      $element['#theme'] = 'links';
      $element['#attributes'] = array('class' => array('inline'));

      $element['#links'][] = array(
        'href' => $uri['path'],
        'attributes' => array('target' => '_blank'),
        'title' => $item->getSourceLabel(),
      );
      return drupal_render($element);
    }
    else {
      return $item->getSourceLabel();
    }
  }

}

/**
 * Extended class.
 */
// @codingStandardsIgnoreStart
// Using same prefix as module name.
class tmgmt_cdt_handler_field_tmgmt_job_item_cdt_source_language extends views_handler_field_entity {
// @codingStandardsIgnoreEnd

  /**
   * Render.
   */
  public function render($values) {
    $item = $this->get_value($values);
    return tmgmt_language_label($item->cdt_source_language);
  }

}

/**
 * Extended class.
 */
// @codingStandardsIgnoreStart
// Using same prefix as module name.
class tmgmt_cdt_handler_field_tmgmt_job_item_cdt_target_language extends views_handler_field_entity {
// @codingStandardsIgnoreEnd

  /**
   * Render.
   */
  public function render($values) {
    $item = $this->get_value($values);
    return tmgmt_language_label($item->cdt_target_language);
  }

}

/**
 * Views controller class for the job item entity.
 */
class TmgmtCdtJobItemViewsController extends TMGMTJobItemViewsController {

  /**
   * {@inheritdoc}
   */
  // @codingStandardsIgnoreStart
  // Need the same name to override.
  public function views_data() {
  // @codingStandardsIgnoreEnd
    $data = parent::views_data();

    $data['tmgmt_job_item']['operations']['field']['handler'] = 'tmgmt_cdt_handler_field_tmgmt_job_item_operations';
    $data['tmgmt_job_item']['label']['field']['handler'] = 'tmgmt_cdt_handler_field_tmgmt_job_item_label';
    $data['tmgmt_job_item']['cdt_source_language']['field']['handler'] = 'tmgmt_cdt_handler_field_tmgmt_job_item_cdt_source_language';
    $data['tmgmt_job_item']['cdt_target_language']['field']['handler'] = 'tmgmt_cdt_handler_field_tmgmt_job_item_cdt_target_language';

    return $data;
  }

}
