<?php

/**
 * @file
 * Implements TMGMTFileFormatInterface.
 */

/**
 * Extend class XMLWriter.
 */
class TmgmtCdtFileFormat extends XMLWriter {

  /**
   * Contains a reference to the currently being exported job.
   *
   * @var TMGMTJob
   */
  protected $job;

  protected $importedXML;
  protected $importedTransUnits;

  /**
   * Store mapped Job Item Ids [lang][entity_type][entity_id].
   *
   * @var array
   */
  public $mappedItemsIDs = array();

  /**
   * Adds a job item to the xml export.
   */
  protected function addItem(TMGMTJobItem $item, TMGMTJob $job) {

    $this->startElement('Translation');
    $this->writeElement('PreviewLink', 'NO PREVIEW LINK');
    $this->writeElement('SourceReference', TmgmtCdtFileFormatHelper::itemSourceReference($item)['link']);

    $data = array_filter(tmgmt_flatten_data($item->getData()), '_tmgmt_filter_data');

    // Find field(s) with "title" and put at the start.
    $titles = array();
    foreach ($data as $key => $field) {
      if (strpos(drupal_strtolower($key), 'title') !== FALSE) {
        $titles[$key] = $data[$key];
        unset($data[$key]);
      }
    }
    $data_ordered = array();
    foreach ($titles as $key => $field) {
      $data_ordered[$key] = $field;
    }
    foreach ($data as $key => $field) {
      $data_ordered[$key] = $field;
    }
    $item->cdt_settings['tagsTypeCount'] = array();

    foreach ($data_ordered as $key => $field) {
      $this->startElement('StaticContent');

      $this->writeAttribute('indexType', $item->item_type);
      $this->writeAttribute('instanceId', $item->item_id);
      $this->writeAttribute('resName', '][' . $key);
      $this->writeAttribute('resLabel', @$field['#label']);
      $content = TmgmtCdtFileFormatHelper::convertDrupalToToken(trim($field['#text']));

      $_key = str_replace('[', '', $key);
      $_key = str_replace(']', '', $_key);
      $item->cdt_settings['tagsTypeCount'][$_key] = TmgmtCdtFileFormatHelper::tagsTypeCount($content);

      $item->save();
      $this->writeCdata($content);
      unset($content);
      // End tag </StaticContent>.
      $this->endElement();
    }

    // End tag </Translation>.
    $this->endElement();
  }

  /**
   * {@inheritdoc}
   */
  public function export(TMGMTJob $job) {
    $this->job = $job;

    $this->openMemory();
    $this->setIndent(TRUE);
    $this->setIndentString(' ');
    $this->startDocument('1.0', 'UTF-8', 'no');

    // <Transaction> Root element.
    $this->startElement('Transaction');
    $this->writeAttribute('xmlns:xsi', 'http://www.w3.org/2001/XMLSchema-instance');
    $this->writeAttribute('xsi:noNamespaceSchemaLocation', 'http://static.cdt.europa.eu/webtranslations/schemas/Drupal-Translation-V7-1.xsd');

    $this->startElement('TransactionHeader');
    $this->startElement('SenderDetails');
    $this->writeElement('ProducerDateTime', date('c', $job->created));
    // End tag </SenderDetails>.
    $this->endElement();
    // End tag </TransactionHeader>.
    $this->endElement();

    $this->writeElement('TransactionIdentifier', $job->tjid);
    $this->writeElement('TransactionCode', 'Drupal Translation Request');

    // End tag <TransactionData>.
    $this->startElement('TransactionData');
    $this->startElement('TranslationDetails');

    $items_unique = array();
    foreach ($job->getItems() as $item) {
      if (in_array($item->item_id, $items_unique)) {
        continue;
      }
      $this->addItem($item, $job);
      $items_unique[] = $item->item_id;
    }
    unset($items_unique);
    $this->writeElement('TotalCharacterLength', $job->getCounts()['characters']);
    // End tag </TransactionDetails>.
    $this->endElement();
    // End tag </TransactionData>.
    $this->endElement();

    $this->endDocument();
    return $this->outputMemory();
  }

  /**
   * Check xml.
   */
  public function validateImport($xml_string) {
    try {

      $xml = simplexml_load_string($xml_string);
      if ($xml === FALSE) {
        throw new \Exception(t('Error parsing xml'));
      }

      $xml->registerXPathNamespace('Transaction', 'http://static.cdt.europa.eu/webtranslations/schemas/Drupal-Translation-V7-1.xsd');
      $transaction_identifier = $xml->xpath('//TransactionIdentifier');

      if ($transaction_identifier) {
        $transaction_identifier = reset($transaction_identifier);
      }
      else {
        throw new \Exception(t('TransactionIdentifier tag is missing'));
      }

      $tjid = (int) $transaction_identifier;
      $job = tmgmt_job_load($tjid);
      if (empty($job)) {
        throw new \Exception(t('Cannot find matching job ID: @tjid in the system', array('@tjid' => $tjid)));
      }

      unset($xml);
      return TRUE;
    }
    catch (\Exception $e) {
      if (isset($xml)) {
        unset($xml);
      }
      drupal_set_message($e->getMessage(), 'error');
      return FALSE;
    }
  }

  /**
   * Import xml.
   */
  public function import($xml_string, $target_language) {

    $this->importedXml = simplexml_load_string($xml_string);
    if ($this->importedXml === FALSE) {
      throw new \Exception(t('Error parsing xml'));
    }

    $this->importedXml->registerXPathNamespace('Transaction', 'http://static.cdt.europa.eu/webtranslations/schemas/Drupal-Translation-V7-1.xsd');
    $transaction_identifier = $this->importedXml->xpath('//TransactionIdentifier');

    if ($transaction_identifier) {
      $transaction_identifier = reset($transaction_identifier);
    }
    else {
      throw new \Exception(t('TransactionIdentifier tag is missing'));
    }

    $tjid = (int) $transaction_identifier;
    $job = tmgmt_job_load((int) $tjid);

    if (empty($job)) {
      throw new \Exception(t('Cannot find inside the system a matching job ID: @tjid', array('@tjid' => $tjid)));
    }

    $flat_data = $this->getImportedTargets($job, $target_language);
    $data = tmgmt_unflatten_data($flat_data);

    return $data;
  }

  /**
   * Import per target language.
   */
  protected function getImportedTargets(TMGMTJob $job, $target_language) {
    $this->getMappedItemIds($job);
    $importedTransUnits = array();

    $errors = 0;
    $warnings = 0;
    $reader = new XMLReader();

    foreach ($this->importedXml->xpath('//Translation') as $translation) {
      if ($job->source_language != $target_language) {
        foreach ($translation->StaticContent as $unit) {
          $entity_id = (string) $unit['instanceId'];
          $entity_type = (string) $unit['indexType'];

          if (!empty((string) $unit['resName'])) {
            $entity_field = (string) $unit['resName'];
          }
          else {
            $entity_field = (string) $unit['name'];
          }

          if (!empty($this->mappedItemsIDs[$target_language][$entity_type][$entity_id])) {

            $job_items = $job->getItems(array('item_id' => $entity_id, 'cdt_target_language ' => $target_language));
            $job_item = reset($job_items);

            try {

              if (variable_get('tmgmt_cdt_html_check', TMGMT_CDT_HTML_CHECK)) {

                $entity_field_clean = str_replace('[', '', $entity_field);
                $entity_field_clean = str_replace(']', '', $entity_field_clean);

                $source_tags = @$job_item->cdt_settings['tagsTypeCount'][$entity_field_clean];
                $target_tags = TmgmtCdtFileFormatHelper::tagsTypeCount((string) $unit);

                // Check drupal-token (cannot import translation)
                if (isset($source_tags['drupal-token'])) {
                  if ($source_tags['drupal-token'] !== $target_tags['drupal-token']) {
                    throw new Exception(t('Drupal-token tags in the source and target do not match'));
                  }
                }
                if (isset($source_tags['/drupal-token'])) {
                  if ($source_tags['/drupal-token'] !== $target_tags['/drupal-token']) {
                    throw new Exception(t('/Drupal-token tags in the source and target do not match'));
                  }
                }

                // Check tags quantity (can import translation)
                if (count($source_tags) !== count($target_tags)) {
                  $warnings++;
                  $job_item->addMessage(t('The quantity of tags in the source and the target is different (@language)'), array('@language' => $target_language), 'warning');
                }

              }

              $item_id = $this->mappedItemsIDs[$target_language][$entity_type][$entity_id];
              $reader->XML($unit->asXML());
              $key = $item_id . $entity_field;

              $converted_content = TmgmtCdtFileFormatHelper::convertTokenToDrupal((string) $unit);

              if (strpos($converted_content, 'drupal-token') !== FALSE) {
                throw new Exception(t('The content has not been converted. Please check the translation'));
              }

              $importedTransUnits[$key]['#text'] = TmgmtCdtFileFormatHelper::convertTokenToDrupal((string) $unit);
            }
            catch (\Exception $e) {
              $errors++;
              $job_item->addMessage('ERROR @message (@language)', array('@message' => $e->getMessage(), '@language' => $target_language), 'error');
            }

          }
        }
      }
    }

    if ($warnings > 0) {
      drupal_set_message(t('The import has generated @warnings warnings (but translations are imported). Please consult the message list (at the bottom of the job page)', array('@warnings' => $warnings)), 'warning', FALSE);
    }

    if ($errors > 0) {
      drupal_set_message(t('The import has generated  @errors errors (translations are not imported). Please consult the message list (at the bottom of the job page)', array('@errors' => $errors)), 'error', FALSE);
    }
    return $importedTransUnits;
  }

  /**
   * Job Item ids are stored in [lang][entity_type][entity_id].
   */
  public function getMappedItemIds(TMGMTJob $job) {
    $items = $job->getItems();
    foreach ($items as $item_id => $item) {
      $this->mappedItemsIDs[$item->cdt_target_language][$item->item_type][$item->item_id] = $item_id;
    }
  }

}

/**
 * Helper.
 */
class TmgmtCdtFileFormatHelper {

  /**
   * ConvertTokenToDrupal.
   */
  public static function convertTokenToDrupal($html) {

    $item = self::getToken($html);

    $count = 0;
    while ($item !== FALSE) {
      $count++;

      $token = TmgmtCdtFileFormatHelper::extractTokenData($item);
      $new_tag = TmgmtCdtFileFormatHelper::restoreToken($token);
      $token_html = TmgmtCdtFileFormatHelper::getTagByEncoded($html, $token['encoded']);
      $html = str_replace($token_html, $new_tag, $html);

      $item = TmgmtCdtFileFormatHelper::getToken($html);

      // Anti infinite loop ...
      if ($count > 3000) {
        $item = FALSE;
      }
    }
    return $html;

  }

  /**
   * To drupal.
   */
  public static function convertDrupalToToken($html) {

    // Correct order script>drupal>img>a.
    if (variable_get('tmgmt_cdt_use_token_html_script', TMGMT_CDT_USE_TOKEN_HTML_SCRIPT) === TRUE) {
      $list = TmgmtCdtFileFormatHelper::getTagsHtmlScript($html);
      foreach ($list as $key => $item) {
        $token_html = TmgmtCdtFileFormatHelper::encodeToken($item);
        $html = str_replace($item['original_html'], $token_html, $html);
      }
      unset($key);
    }

    if (variable_get('tmgmt_cdt_use_token_drupal', TMGMT_CDT_USE_TOKEN_DRUPAL) === TRUE) {
      $list = TmgmtCdtFileFormatHelper::getTagsDrupal($html);
      foreach ($list as $key => $item) {
        $token_html = TmgmtCdtFileFormatHelper::encodeToken($item);
        $html = str_replace($item['original_html'], $token_html, $html);
      }
    }

    if (variable_get('tmgmt_cdt_use_token_html_img', TMGMT_CDT_USE_TOKEN_HTML_IMG) === TRUE) {
      $list = TmgmtCdtFileFormatHelper::getTagsHtmlImg($html);
      foreach ($list as $key => $item) {
        $token_html = TmgmtCdtFileFormatHelper::encodeToken($item);
        $html = str_replace($item['original_html'], $token_html, $html);
      }
    }

    if (variable_get('tmgmt_cdt_use_token_html_a', TMGMT_CDT_USE_TOKEN_HTML_A) === TRUE) {
      $list = TmgmtCdtFileFormatHelper::getTagsHtmlA($html);
      foreach ($list as $key => $item) {
        $token_html = TmgmtCdtFileFormatHelper::encodeToken($item);
        $html = str_replace($item['original_html'], $token_html, $html);
      }
    }
    return $html;
  }

  /**
   * Find Tokens.
   */
  public static function getToken($html) {
    preg_match_all('/<drupal-token ((?!<drupal-token ).)*?<\/drupal-token>/ims', $html, $finds, PREG_SET_ORDER, 0);
    if (count($finds) > 0) {
      preg_match_all('/<drupal-token ((?!<drupal-token ).)*?<\/drupal-token>/ims', drupal_substr($finds[0][0], 13, -13), $sub_finds, PREG_SET_ORDER, 0);
      if (count($sub_finds) > 0) {
        return TmgmtCdtFileFormatHelper::getToken($sub_finds[0][0]);
      }
      return $finds[0][0];
    }
    else {
      return FALSE;
    }
  }

  /**
   * Extract.
   */
  public static function extractTokenData($token_html) {
    $token = array(
      'token_html' => $token_html,
      'encoded' => '',
      'original_html' => '',
      'type' => '',
      'items' => array(),
    );
    preg_match_all('/encoded=[\"\'](.*?)[\"\']/im', $token_html, $finds, PREG_SET_ORDER, 0);
    if (isset($finds[0][1])) {
      $token['encoded'] = $finds[0][1];
      $token['original_html'] = base64_decode($finds[0][1]);
    }
    preg_match_all('/type=[\"\'](.*?)[\"\']/im', $token_html, $finds, PREG_SET_ORDER, 0);
    if (isset($finds[0][1])) {
      $token['type'] = $finds[0][1];
    }
    preg_match_all('/<drupal-token-text[\s]+([^>]+)>((?:.(?!\<\/drupal-token-text\>))*.)<\/drupal-token-text>/ims', $token_html, $matchs, PREG_SET_ORDER, 0);
    foreach ($matchs as $id => $item) {
      $token['items'][explode('"', $item[1])[1]] = $item[2];
    }
    unset($id);
    return $token;
  }

  /**
   * RestoreDrupalMediaToken.
   */
  public static function restoreDrupalMediaToken($token) {
    $content = drupal_substr($token['original_html'], 2);
    $content = drupal_substr($content, 0, -2);
        // @codingStandardsIgnoreStart
        // Need native json format.
        $content = json_decode($content, TRUE);
        foreach ($token['items'] as $path => $value) {
            TmgmtCdtFileFormatHelper::arraySetValue($content, $path, $value);
        }
        // Need native json format
        $content = '[[' . json_encode($content, JSON_HEX_TAG | JSON_HEX_APOS | JSON_HEX_AMP | JSON_HEX_QUOT) . ']]';
        // @codingStandardsIgnoreEnd
    return $content;
  }

  /**
   * RestoreDrupalSimpleToken.
   */
  public static function restoreDrupalSimpleToken($token) {
    return $token['original_html'];
  }

  /**
   * RestoreHtmlScript.
   */
  public static function restoreHtmlScript($token) {
    return $token['original_html'];
  }

  /**
   * RestoreHtmlA.
   */
  public static function restoreHtmlA($token) {
    $doc = new DomDocument('1.0', 'UTF-8');
    libxml_use_internal_errors(TRUE);
    $doc->loadHTML($token['original_html']);
    $tag = $doc->getElementsByTagName('a')->item(0);

    foreach ($token['items'] as $path => $value) {
      if ($path === 'label') {
        $tag->nodeValue = '';
        $fragment = $doc->createDocumentFragment();
        $fragment->appendXML(html_entity_decode($value));
        $tag->appendChild($fragment);
      }
      else {
        $tag->setAttribute($path, html_entity_decode($value));
      }
    }
    return $tag->ownerDocument->saveXML($tag);
  }

  /**
   * RestoreHtmlImg.
   */
  public static function restoreHtmlImg($token) {
    $doc = new DomDocument('1.0', 'UTF-8');
    libxml_use_internal_errors(TRUE);
    $doc->loadHTML($token['original_html']);
    $tag = $doc->getElementsByTagName('img')->item(0);
    foreach ($token['items'] as $path => $value) {
      $tag->setAttribute($path, $value);
    }
    return $tag->ownerDocument->saveXML($tag);
  }

  /**
   * RestoreToken.
   */
  public static function restoreToken($token) {
    $html_tag = '';
    switch ($token['type']) {
      case TMGMT_CDT_TOKEN_TYPE_DRUPAL_TOKEN_MEDIA:
        $html_tag = TmgmtCdtFileFormatHelper::restoreDrupalMediaToken($token);
        break;

      case TMGMT_CDT_TOKEN_TYPE_DRUPAL_TOKEN_SIMPLE:
        $html_tag = TmgmtCdtFileFormatHelper::restoreDrupalSimpleToken($token);
        break;

      case TMGMT_CDT_TOKEN_TYPE_HTML_SCRIPT:
        $html_tag = TmgmtCdtFileFormatHelper::restoreHtmlScript($token);
        break;

      case TMGMT_CDT_TOKEN_TYPE_HTML_A:
        $html_tag = TmgmtCdtFileFormatHelper::restoreHtmlA($token);
        break;

      case TMGMT_CDT_TOKEN_TYPE_HTML_IMG:
        $html_tag = TmgmtCdtFileFormatHelper::restoreHtmlImg($token);
        break;
    }
    return $html_tag;
  }

  /**
   * GetTagByEncoded.
   */
  public static function getTagByEncoded($html, $encoded) {
    $original_html = FALSE;
    preg_match_all('/<drupal-token ((?!<drupal-token ).)*?<\/drupal-token>/ims', $html, $matchs, PREG_SET_ORDER, 0);
    foreach ($matchs as $id => $item) {
      preg_match_all('/encoded=[\"\'](.*?)[\"\']/im', $item[0], $finds, PREG_SET_ORDER, 0);
      if ($finds[0][1] === $encoded) {
        $original_html = $item[0];
      }
    }
    unset($id);
    return $original_html;
  }

  /**
   * EncodeToken.
   */
  public static function encodeToken($data) {
    try {
      $token_list = array(TMGMT_CDT_TOKEN_TYPE_DRUPAL_TOKEN_MEDIA, TMGMT_CDT_TOKEN_TYPE_DRUPAL_TOKEN_SIMPLE,
        TMGMT_CDT_TOKEN_TYPE_HTML_A, TMGMT_CDT_TOKEN_TYPE_HTML_IMG, TMGMT_CDT_TOKEN_TYPE_HTML_SCRIPT,
      );
      if (!in_array($data['type'], $token_list)) {
        throw new Exception(t('Unknown tag: @type', array('@type' => $data['type'])));
      }
      $html = '<drupal-token type="' . $data['type'] . '" encoded="' . $data['encoded'] . '">';
      foreach ($data['texts'] as $key => $text) {
        $html .= '<drupal-token-text path="' . $key . '">' . $text . '</drupal-token-text>';
      }
      $html .= '</drupal-token>';
      return $html;
    }
    catch (\Exception $e) {
      throw $e;
    }
  }

  /**
   * From https://stackoverflow.com/questions/9628176/using-a-string-path-to-set-nested-array-data/44189105#44189105.
   */
  public static function arraySetValue(&$data, $parents, $value, $glue = '.') {
    if (!is_array($parents)) {
      $parents = explode($glue, (string) $parents);
    }
    $ref = &$data;
    foreach ($parents as $parent) {
      if (isset($ref) && !is_array($ref)) {
        $ref = array();
      }
      $ref = &$ref[$parent];
    }
    $ref = $value;
  }

  /**
   * GetTagsDrupal.
   */
  public static function getTagsDrupal($html) {
    try {

      $list = array();
      // Media token [[...]].
      preg_match_all("/\[\[.*?\]\]/s", strip_tags($html), $matches, PREG_SET_ORDER, 0);
      foreach ($matches as $id => $value) {
        $original_html = $value[0];
        $encoded = base64_encode($original_html);

        $result = array(
          'original_html' => $original_html,
          'encoded' => $encoded,
          'type' => TMGMT_CDT_TOKEN_TYPE_DRUPAL_TOKEN_MEDIA,
          'title' => 'Drupal media Token',
          'texts' => array(),
        );
        $content = $result['original_html'];
        // Remove [[.
        $content = drupal_substr($content, 2);
        // Remove ]].
        $content = drupal_substr($content, 0, -2);
                // @codingStandardsIgnoreStart
                // Need native json format.
                $contentObject = json_decode($content);
                // @codingStandardsIgnoreEnd
        if (isset($contentObject->attributes->alt)) {
          $result['texts']['attributes.alt'] = $contentObject->attributes->alt;
        }
        if (isset($contentObject->attributes->title)) {
          $result['texts']['attributes.title'] = $contentObject->attributes->title;
        }
        $list[] = $result;
        $html = str_replace($original_html, '', $html);
      }
      unset($id);

      // Simple token [...].
      preg_match_all("/\[.*?\]/s", strip_tags($html), $matches, PREG_SET_ORDER, 0);
      foreach ($matches as $id => $value) {
        $original_html = $value[0];
        $encoded = base64_encode($original_html);
        $result = array(
          'original_html' => $original_html,
          'encoded' => $encoded,
          'type' => TMGMT_CDT_TOKEN_TYPE_DRUPAL_TOKEN_SIMPLE,
          'title' => 'Drupal Token',
          'texts' => array(),
        );
        $list[] = $result;
        $html = str_replace($original_html, '', $html);
      }

      // Special tags.
      preg_match_all("/(<none>|<front>|<separator>|<nolink>|<minipanel>)/s", $html, $matches, PREG_SET_ORDER, 0);
      foreach ($matches as $id => $value) {
        $original_html = $value[0];
        $encoded = base64_encode($original_html);
        $result = array(
          'original_html' => $original_html,
          'encoded' => $encoded,
          'type' => TMGMT_CDT_TOKEN_TYPE_DRUPAL_TOKEN_SIMPLE,
          'title' => 'Drupal special Tag',
          'texts' => array(),
        );
        $list[] = $result;
        $html = str_replace($original_html, '', $html);
      }

      return $list;
    }
    catch (\Exception $e) {
      throw $e;
    }
  }

  /**
   * GetTagsHtmlA.
   */
  public static function getTagsHtmlA($html) {
    try {
      $list = array();
      preg_match_all('/<a[\s]+([^>]+)>((?:.(?!\<\/a\>))*.)<\/a>/ims', $html, $matchs, PREG_SET_ORDER, 0);
      foreach ($matchs as $id => $item) {
        $original_html = $item[0];
        $encoded = base64_encode($original_html);
        $result = array(
          'original_html' => $original_html,
          'encoded' => $encoded,
          'type' => TMGMT_CDT_TOKEN_TYPE_HTML_A,
          'title' => 'HTML link',
          'texts' => array(
            'label' => trim($item[2]),
          ),
        );
        preg_match_all('/href=[\"\'](.*?)[\"\']/im', $result['original_html'], $finds, PREG_SET_ORDER, 0);
        if (isset($finds[0][1])) {
          $result['texts']['href'] = $finds[0][1];
        }
        preg_match_all('/hreflang=[\"\'](.*?)[\"\']/im', $result['original_html'], $finds, PREG_SET_ORDER, 0);
        if (isset($finds[0][1])) {
          $result['texts']['hreflang'] = $finds[0][1];
        }
        preg_match_all('/lang=[\"\'](.*?)[\"\']/im', $result['original_html'], $finds, PREG_SET_ORDER, 0);
        if (isset($finds[0][1])) {
          $result['texts']['lang'] = $finds[0][1];
        }
        $list[] = $result;
      }
      unset($id);
      return $list;
    }
    catch (\Exception $e) {
      throw $e;
    }
  }

  /**
   * GetTagsHtmlImg.
   */
  public static function getTagsHtmlImg($html) {
    try {
      $list = array();
      preg_match_all('/<img([\w\W]+?)>/ims', $html, $matchs, PREG_SET_ORDER, 0);
      foreach ($matchs as $id => $item) {
        $original_html = $item[0];
        $encoded = base64_encode($original_html);
        $result = array(
          'original_html' => $original_html,
          'encoded' => $encoded,
          'type' => TMGMT_CDT_TOKEN_TYPE_HTML_IMG,
          'title' => 'HTML link',
          'texts' => array(),
        );
        preg_match_all('/alt=[\"\'](.*?)[\"\']/im', $result['original_html'], $finds, PREG_SET_ORDER, 0);
        if (isset($finds[0][1])) {
          $result['texts']['alt'] = $finds[0][1];
        }
        preg_match_all('/title=[\"\'](.*?)[\"\']/im', $result['original_html'], $finds, PREG_SET_ORDER, 0);
        if (isset($finds[0][1])) {
          $result['texts']['title'] = $finds[0][1];
        }
        preg_match_all('/lang=[\"\'](.*?)[\"\']/im', $result['original_html'], $finds, PREG_SET_ORDER, 0);
        if (isset($finds[0][1])) {
          $result['texts']['lang'] = $finds[0][1];
        }
        $list[] = $result;
      }
      unset($id);
      return $list;
    }
    catch (\Exception $e) {
      throw $e;
    }
  }

  /**
   * GetTagsHtmlScript.
   */
  public static function getTagsHtmlScript($html) {
    try {
      $list = array();

      $doc = new DomDocument('1.0', 'UTF-8');
      libxml_use_internal_errors(TRUE);

      $doc->loadHTML($html);
      $tags = $doc->getElementsByTagName("script");
      foreach ($tags as $tag) {
        $original_html = $doc->saveHTML($tag);
        $encoded = base64_encode($original_html);
        $result = array(
          'original_html' => $original_html,
          'encoded' => $encoded,
          'type' => TMGMT_CDT_TOKEN_TYPE_HTML_SCRIPT,
          'title' => 'HTML script',
          'texts' => array(),
        );
        $list[] = $result;
      }
      return $list;
    }
    catch (\Exception $e) {
      throw $e;
    }
  }

  /**
   * Get html tags count.
   */
  public static function tagsCount($html) {
    preg_match_all('/<((?=!\-\-)!\-\-[\s\S]*\-\-|((?=\?)\?[\s\S]*\?|((?=\/)\/[^.\-\d][^\/\]\'"[!#$%&()*+,;<=>?@^`{|}~ ]*|[^.\-\d][^\/\]\'"[!#$%&()*+,;<=>?@^`{|}~ ]*(?:\s[^.\-\d][^\/\]\'"[!#$%&()*+,;<=>?@^`{|}~ ]*(?:=(?:"[^"]*"|\'[^\']*\'|[^\'"<\s]*))?)*)\s?\/?))>/mi', $html, $matchs);
    return count($matchs[0]);
  }

  /**
   * TagsTypeCount.
   */
  public static function tagsTypeCount($html) {
    preg_match_all('/<((?=!\-\-)!\-\-[\s\S]*\-\-|((?=\?)\?[\s\S]*\?|((?=\/)\/[^.\-\d][^\/\]\'"[!#$%&()*+,;<=>?@^`{|}~ ]*|[^.\-\d][^\/\]\'"[!#$%&()*+,;<=>?@^`{|}~ ]*(?:\s[^.\-\d][^\/\]\'"[!#$%&()*+,;<=>?@^`{|}~ ]*(?:=(?:"[^"]*"|\'[^\']*\'|[^\'"<\s]*))?)*)\s?\/?))>/mi', $html, $matchs);
    $list = array();
    foreach (array_values($matchs[1]) as $val) {
      $exp = explode(' ', drupal_strtolower($val));
      if (!isset($list[$exp[0]])) {
        $list[$exp[0]] = 0;
      }
      $list[$exp[0]]++;
    }
    return $list;
  }

  /**
   * ItemSourceReference.
   */
  public static function itemSourceReference(TMGMTJobItem $job_item) {

    $website_url_home = url('<front>', array(
      'absolute' => TRUE,
      'language' => language_list()[$job_item->cdt_source_language],
    ));

    $website_name = trim(variable_get('site_name', 'drupal 7'));

    $item = array(
      'link' => '',
      'label' => '',
    );

    switch ($job_item->item_type) {

      case 'node':
        $node = node_load($job_item->item_id);
        $item['link'] = url('node/' . $job_item->item_id, array(
          'absolute' => TRUE,
          'language' => language_list()[$job_item->cdt_source_language],
        ));
        $item['label'] = $node->title;
        break;

      case 'taxonomy_term':
        $_item_id = explode(':', $job_item->item_id);
        $item_id = $_item_id[(count($_item_id) - 1)];

        $term = taxonomy_term_load($item_id);
        $item['link'] = url('taxonomy/term/' . $item_id, array(
          'absolute' => TRUE,
          'language' => language_list()[$job_item->cdt_source_language],
        ));
        $item['label'] = $term->name;
        break;

      case 'menu_link':
        $_item_id = explode(':', $job_item->item_id);
        $item_id = $_item_id[(count($_item_id) - 1)];
        $menu = menu_link_load($item_id);

        $menu['link_path'] = drupal_get_normal_path($menu['link_path'], $job_item->cdt_source_language);

        if (drupal_substr(drupal_strtolower($menu['link_path']), 0, 4) !== 'http') {
          $item['link'] = $website_url_home;
        }
        else {
          $item['link'] = $menu['link_path'];
        }

        $item['label'] = $menu['link_title'];
        break;

      case 'block':
        $item['link'] = $website_url_home;
        $item['label'] = $website_name;

        break;

      case 'views':
        $item['link'] = $website_url_home;
        $item['label'] = $website_name;

        break;

      case 'default':
        $item['link'] = $website_url_home;
        $item['label'] = $website_name;
        break;
    }

    return $item;

  }

}
