<?php

/**
 * @file
 * TMGMT plugin class.
 */

/**
 * Override TMGMTDefaultTranslatorPluginController.
 */
class TMGMTCdtPluginController extends TMGMTDefaultTranslatorPluginController {

  /**
   * Override isAvailable().
   */
  public function isAvailable(TMGMTTranslator $translator) {
    return TRUE;
  }

  /**
   * Disable Cancel action.
   */
  public function abortTranslation(TMGMTJob $job) {
    drupal_set_message(t('Please contact the @cdt_name to cancel a translation task', array('@cdt_name' => TMGMT_CDT_NAME)), 'warning', FALSE);
    return FALSE;
  }

  /**
   * Override canTranslate().
   */
  public function canTranslate(TMGMTTranslator $translator, TMGMTJob $job) {
    $errors = array();
    try {
      $job_items = $job->getItems();

      if (!in_array($translator->mapToRemoteLanguage($job->source_language), $this->getApiLanguages())) {
        $errors[] = t('Cannot translate source language code: @source_language', array('@source_language' => $job->source_language));
      }

      $job_check = _tmgmt_cdt_check_job($job);
      if (count($job_check) > 0) {
        $errors = array_merge($errors, $job_check);
      }

      $count = 0;
      foreach ($job_items as $item) {
        $count++;

        // cdt_target_language is set by cdt translator. before is empty.
        if (!empty($item->cdt_target_language) && !in_array($translator->mapToRemoteLanguage($item->cdt_target_language), $this->getApiLanguages())) {
          $errors[] = t('Cannot translate target language code: @cdt_target_language', array('@cdt_target_language' => $item->cdt_target_language));
        }

        $job_item_check = _tmgmt_cdt_check_job_item($item);
        if (count($job_item_check) > 0) {
          $errors = array_merge($errors, $job_item_check);
        }

      }

      if (count($errors) > 0) {
        throw new \Exception();
      }

      return TRUE;
    }
    catch (\Exception $e) {
      foreach ($errors as $error) {
        drupal_set_message(check_plain($error), 'error', FALSE);
      }
      return FALSE;
    }
  }

  /**
   * Override hasCheckoutSettings().
   */
  public function hasCheckoutSettings(TMGMTJob $job) {
    return TRUE;
  }

  /**
   * Override requestTranslation().
   */
  public function requestTranslation(TMGMTJob $job) {

    try {
      global $user;

      $file_name = 'cdt_job_' . $job->tjid . '_request.xml';
      $file_format = new TmgmtCdtFileFormat();
      $file_path = "public://tmgmt_file/$file_name";

      $dirname = drupal_dirname($file_path);
      if (!file_prepare_directory($dirname, FILE_CREATE_DIRECTORY)) {
        throw new \Exception(t('Cannot write to folder: @dirname', array('@dirname' => $dirname)));
      }

      $xml = $file_format->export($job);

      if (!$xml) {
        throw new \Exception(t('Cannot generate xml data from job'));
      }

      $file_status = 'create';

      if (file_exists($file_path)) {
        $file_status = 'update';
      }

      $file_xml = file_save_data($xml, $file_path, FILE_EXISTS_REPLACE);
      if (!$file_xml) {
        throw new \Exception(t('Cannot create or update xml file: @file_path', array('@file_path' => $file_path)));
      }

      switch ($file_status) {
        case 'create':
          file_usage_add($file_xml, 'tmgmt_file', 'tmgmt_job', $job->tjid);
          $job->addMessage(
          'Translation <a href="@job_url">job</a> file was created. <a target="_blank" href="@link">Click here to download the XML file</a>',
          array('@link' => file_create_url($file_xml->uri), '@job_url' => url("admin/tmgmt/jobs/" . $job->tjid))
          );
          break;

        default:
          $job->addMessage(
          'Translation <a href="@job_url">job</a> file was updated. <a target="_blank" href="@link">Click here to download the XML file</a>',
          array('@link' => file_create_url($file_xml->uri), '@job_url' => url("admin/tmgmt/jobs/" . $job->tjid))
          );

          break;
      }

      $request_params = $this->generateRequest($job, $xml);

      // Save now to have data for debug.
      $job->cdt_request_params = drupal_json_encode($request_params);
      $job->target_language = 'multiple';
      $job->save();

      $api = TmgmtCdtApi::getInstance();
      $validate_request = $api->validateRequest($request_params);
      if (!$validate_request) {
        throw new \Exception(t('Cannot validate request from @cdt_name', array(' @cdt_name' => TMGMT_CDT_NAME)));
      }
      $job->jobItemGenerateTargetLanguages();

      $api = TmgmtCdtApi::getInstance();
      $api->setParamsBody($request_params);
      $api_result = $api->post('Requests');

      if ($api_result->status !== 'success') {
        throw new \Exception($api_result->error);
      }

      $job->submitted(t('Job has been submitted to @cdt_name (Correlation ID: @correlation_id)', array('@cdt_name' => TMGMT_CDT_NAME, '@correlation_id' => trim($api_result->content))));

      $job->cdt_correlation_id = trim($api_result->content, '"');
      $job->cdt_request_identifier = TMGMT_CDT_DEFAUTL_REQUEST_IDENTIFIER;
      $job->cdt_request_params = drupal_json_encode($request_params);
      $job->save();

      $user_data = entity_metadata_wrapper('user', $user);
      $user_cdt_default = array(
        'cdt' => array(
          'priority' => $job->getSetting('cdt')['priority'],
          'confidentiality' => $job->getSetting('cdt')['confidentiality'],
          'department' => $job->getSetting('cdt')['department'],
          'contacts' => $job->getSetting('cdt')['contacts'],
          'deliver_to' => $job->getSetting('cdt')['deliver_to'],
          'phone' => $job->getSetting('cdt')['phone'],
        ),
      );

      $user_data->field_cdt_default->set(serialize($user_cdt_default));
      $user_data->save();

      return TRUE;
    }
    catch (\Exception $e) {
      watchdog_exception('tmgmt_cdt', $e);
      drupal_set_message($e->getMessage(), 'error');
      $job->rejected('Job has been rejected with following error: @error', array('@error' => $e->getMessage()), 'error');
    }
  }

  /**
   * Return languages from API.
   */
  private function getApiLanguages() {
    try {

      $api = TmgmtCdtApi::getInstance();
      $business_reference = $api->businessReferenceData();

      if (!isset($business_reference->Languages)) {
        return array();
      }
      $languages = $business_reference->Languages;
      asort($languages);

      return $languages;
    }
    catch (\Exception $e) {
      drupal_set_message($e->getMessage(), 'error');
      return array();
    }
  }

  /**
   * Override getSupportedRemoteLanguages().
   */
  public function getSupportedRemoteLanguages(TMGMTTranslator $translator) {
    try {
      $languages = $this->getApiLanguages();

      return array_combine($languages, $languages);
    }
    catch (\Exception $e) {
      return array();
    }
  }

  /**
   * Generate parameters for the request API.
   */
  private function generateRequest(TMGMTJob $job, $xml_data) {

    $job_settings = $job->getSetting('cdt');

    $params = new stdClass();
    $params->Department = new stdClass();
    $params->Department->Code = $job_settings['department'];

    $params->Contacts = array();

    foreach ($job_settings['contacts'] as $contact) {
      $_contact = new stdClass();
      $_contact->UserName = $contact;
      $params->Contacts[] = $_contact;
      unset($_contact);
    }

    $params->DeliveryContacts = array();

    foreach ($job_settings['deliver_to'] as $contact) {
      $_contact = new stdClass();
      $_contact->UserName = $contact;
      $params->DeliveryContacts[] = $_contact;
      unset($_contact);
    }

    $phone = preg_replace('/\D/', '', $job_settings['phone']);
    $phone = drupal_substr($phone, 0, 15);
    $params->PhoneNumber = $phone;

    $params->Title = 'Translation ' . $job->tjid . ' for ' . trim(variable_get('site_name', 'drupal 7'));

    $params->ClientReference = $job->tjid;

    $params->Purpose = new stdClass();
    $params->Purpose->Code = 'WS';

    $params->Priority = new stdClass();
    $params->Priority->Code = $job_settings['priority'];

    $params->TranslationsWebUploaded = new stdClass();
    $params->TranslationsWebUploaded->Code = 'No';

    $comments = '';

    $comments .= $job_settings['options'] . "\n";

    foreach ($job_settings['related_jobs'] as $related_job) {
      $comments .= "RELATED_JOB: $related_job\n";
    }

    $comments .= check_plain($job_settings['comments']);

    $params->Comments = $comments;
    unset($comments);

    $request_references = array();
    $job_items = $job->getItems();

    $items_unique = array();

    $translator = $job->getTranslator();

    foreach ($job_items as $item) {
      if (in_array($item->item_id, $items_unique)) {
        continue;
      }
      if ($item->cdt_source_language !== $job->source_language) {
        continue;
      }
      $ref = new stdClass();
      $ref->ReferenceLanguages = array($translator->mapToRemoteLanguage($job->source_language));

      $item_meta = TmgmtCdtFileFormatHelper::itemSourceReference($item);

      $ref->Url = $item_meta['link'];
      $ref->ShortName = $item_meta['label'];

      $request_references[] = $ref;
      $items_unique[] = $item->item_id;
    }

    unset($items_unique);

    $params->RequestReferences = $request_references;
    unset($request_references);

    $params->SourceDocuments = array();
    $params->SourceDocuments[0] = new stdClass();
    $params->SourceDocuments[0]->File = new stdClass();
    $params->SourceDocuments[0]->File->FileDataBase64 = base64_encode($xml_data);
    $params->SourceDocuments[0]->File->FileName = 'translation_job_' . $job->tjid . '_request.xml';
    $params->SourceDocuments[0]->File->TempFileName = '';
    $params->SourceDocuments[0]->File->ReferenceLanguages = array(
      $translator->mapToRemoteLanguage($job->source_language),
    );

    $params->SourceDocuments[0]->SourceLanguages = array(
      $translator->mapToRemoteLanguage($job->source_language),
    );
    $params->SourceDocuments[0]->OutputFormat = new stdClass();
    $params->SourceDocuments[0]->OutputFormat->Code = 'XM';
    $params->SourceDocuments[0]->TranslationJobs = array();

    $counts = $job->getCounts();
    foreach ($job_settings['target_languages'] as $id => $lang) {
      $_job = new stdClass();
      $_job->Volume = $counts['pages_language'];
      $_job->SourceLanguage = $translator->mapToRemoteLanguage($job->source_language);
      $_job->TargetLanguage = $translator->mapToRemoteLanguage($lang);
      $params->SourceDocuments[0]->TranslationJobs[] = $_job;
    }
    unset($id);

    $params->SourceDocuments[0]->Confidentiality = new stdClass();
    $params->SourceDocuments[0]->Confidentiality->Code = $job_settings['confidentiality'];

    $params->SendOptions = 1;
    $params->Service = 0;

    return $params;
  }

  /**
   * Check params bettwen request and response.
   */
  public function validateRequestResponse(TMGMTJob $job, $response) {

    if (count($response) !== count($job->settings['cdt']['target_languages'])) {
      throw new Exception(t('Request and response do not have the same number of target languages'));
    }
  }

}
